@@ -2,7 +2,12 @@ from .climate import *
 from .proxy import *
 from .da import *
 from .reconjob import *
-from . import utils, ml
+from .reconres import *
+try:
+    from . import ml
+except:
+    pass
+
 from .visual import (
     set_style,
     showfig,
