@@ -31,6 +31,7 @@ class PAGES2k:
         'coral.calc': sns.xkcd_rgb['yellow'],
         'coral.SrCa': sns.xkcd_rgb['orange'],
         'coral.d18O': sns.xkcd_rgb['amber'],
+        'coral.d18Osw': sns.xkcd_rgb['light yellow'],
         'ice.melt': sns.xkcd_rgb['pale blue'],
         'ice.d18O': sns.xkcd_rgb['light blue'],
         'ice.dD': sns.xkcd_rgb['sky blue'],
@@ -68,6 +69,7 @@ class PAGES2k:
         'coral.calc': 'P',
         'coral.SrCa': 'X',
         'coral.d18O': 'o',
+        'coral.d18Osw': 'H',
         'ice.melt': '<',
         'ice.d18O': 'd',
         'ice.dD': '>',
